﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using System.Text;

//  ProfitUI Quick Raid Buttons tool
//  Copyright (C) 2012 Todd T Knarr <tknarr@silverglass.org> <tknarr@cox.net> <todd.knarr@gmail.com>

//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

namespace QuickRaidButtons
{
    // The button assignment window form. This is the top-level window for the application. We create it passing in a
    // reference to a UICode object to be used for generating the UI code and to set things like how many buttons
    // there are. Right now that's a formality since the window is hard-coded for 5 button fields, the number ProfitUI
    // supports. A better way would be to have the constructor create the UI fields for button assignments based on
    // the number of buttons and adjust the window size and UI element positioning to accomodate the button assignement
    // elements. That, though, is for another day.
    public partial class ButtonAssignment : Form
    {
        private UICode uicode = null; // Reference to the object to use to format/write the UI code file
        private QRBStorage qrb = null; // Reference to our persistent storage object
        private int button_count = 5; // Default number of quick-raid buttons\
        private string HelpFile = "";

        private string currentButtonClass = ""; // The current class radio button selection
        private Dictionary<string, QRBButtonAssignment> buttonAssignments = null; // Our current button assignment state reflecting the GUI state

        // Retrieve environment and application settings information and create our QRBStorage object. It'll
        // load itself from the storage file as it's created.
        private QRBStorage createQRBStorage()
        {
            StringBuilder filePath = new StringBuilder();
            filePath.Append( Environment.GetFolderPath( Environment.SpecialFolder.ApplicationData ) );
            filePath.Append( Path.DirectorySeparatorChar );
            filePath.Append( Properties.Settings.Default.StorageFolder );
            string StoragePath = filePath.ToString();
            if ( !Directory.Exists( StoragePath ) )
                Directory.CreateDirectory( StoragePath );
            filePath.Append( Path.DirectorySeparatorChar );
            filePath.Append( Properties.Settings.Default.StorageFilename );
            string StorageFilename = filePath.ToString();
            Debug.WriteLine( "Storage file name: " + StorageFilename );
            QRBStorage qrb = new QRBStorage( StorageFilename, button_count );

            return qrb;
        }

        // Populate the radio buttons. We need to do this when we start the program, at this point none of the
        // buttons will be selected.
        private void populateRadioButtons()
        {
            Control[] panel = Controls.Find( "panel1", true );
            if ( panel.Length != 0 )
            {
                foreach ( Control c in panel[0].Controls )
                {
                    if ( c.Name.StartsWith( "radioButton" ) )
                    {
                        RadioButton rb = c as RadioButton;
                        if ( rb != null )
                        {
                            if ( String.Equals( rb.Text, currentButtonClass ) )
                                rb.Checked = true;
                            else
                                rb.Checked = false;
                        }
                    }
                }
            }
        }

        // Populate the UI form fields based on the current state. We locate the current class's button assignments, creating a
        // blank object and adding it to the collection if we couldn't find one, and the current class's spell lines. For each
        // button assignment combo box, we populate it in with a blank item and then one item for each spell line for the class
        // (they should be in alphabetical order already). If the item we're adding matches the current spell line assigned to
        // this button, we remember the item's index so we can set that as the initial selection for the combo box. If we don't
        // find a match we'll set the selection to the blank first item (nothing assigned to this button).
        private void populateButtonAssignments()
        {
            // Locate the button assignments for the current class
            QRBButtonAssignment currentAssignments = null;
            if ( buttonAssignments.ContainsKey( currentButtonClass ) )
                currentAssignments = buttonAssignments[currentButtonClass];
            if ( currentAssignments == null )
            {
                currentAssignments = new QRBButtonAssignment( currentButtonClass, button_count );
                buttonAssignments[currentButtonClass] = currentAssignments;
            }

            // Locate the spell lines for the current class
            QRBSpellClass spellLines = null;
            if ( qrb.SpellClasses.ContainsKey( currentButtonClass ) )
                spellLines = qrb.SpellClasses[currentButtonClass];

            // Populate the pull-down boxes for the 5 buttons we support
            int buttonNumber = 0;
            ComboBox[] cbs = new ComboBox[] { comboBox1, comboBox2, comboBox3, comboBox4, comboBox5 };
            foreach ( ComboBox cb in cbs )
            {
                buttonNumber++;
                Debug.WriteLine( "Button number " + buttonNumber.ToString() );
                int buttonIndex = 0;
                string currentButton = currentAssignments.getButton( buttonNumber );
                cb.Items.Clear();
                // We always have a blank no-assignment possibility
                cb.Items.Add( "" );
                if ( spellLines != null )
                {
                    int i = 1; // Remember that we have that initial blank entry in the combobox
                    foreach ( QRBSpellLine sl in spellLines.SpellLines )
                    {
                        Debug.WriteLine( "Spell line " + i.ToString() + ": " + sl.Linename );
                        cb.Items.Add( sl.Linename );
                        // If the entry we just populated matches the current button assignment, remember
                        // it's index so we can set the drop-down's initial value
                        if ( sl.Linename == currentButton )
                            buttonIndex = i;
                        i++;
                    }
                }

                Debug.WriteLine( "Active spell line: " + buttonIndex.ToString() );
                // Set the initial item in the drop-down to the current selection, or to the
                // blank line if we didn't have a selection.
                cb.SelectedIndex = buttonIndex;
                //cb.SelectedText = currentButton;
            }
        }

        // Our primary constructor. A UICode object is required. We create our QRBStorage object and
        // set the initial class based on the saved application setting for it. If we don't have an
        // initial class, start with Guardian. Once we're done with all that, we create our current
        // button assignment state using a cloned copy of the permanent state and populate our UI
        // fields with the data from the current state.
        public ButtonAssignment( UICode uic )
        {
            InitializeComponent();

            // Conditional check. In development the VS project has the help file one level down in the Help Files folder. In the installer and
            // zipfile versions, the help file only exists in the directory the executable's in. So in debug builds we check for the project folder
            // first, then the executable folder. In release builds we assume we aren't in the VS project environment and only check the executable
            // folder.
#if DEBUG
            HelpFile = Path.GetDirectoryName( Application.ExecutablePath ) + Path.DirectorySeparatorChar + "Help Files" + Path.DirectorySeparatorChar + "QuickRaidButtons.chm";
            if ( !File.Exists( HelpFile ) )
                HelpFile = Path.GetDirectoryName( Application.ExecutablePath ) + Path.DirectorySeparatorChar + "QuickRaidButtons.chm";
#else
            HelpFile = Path.GetDirectoryName( Application.ExecutablePath ) + Path.DirectorySeparatorChar + "QuickRaidButtons.chm";
#endif

            uicode = uic;
            button_count = uicode.ButtonCount;

            qrb = createQRBStorage();
            buttonAssignments = qrb.CloneButtonAssignments();
            if ( !String.IsNullOrEmpty( Properties.Settings.Default.InitialClass ) )
                currentButtonClass = Properties.Settings.Default.InitialClass;
            else
                currentButtonClass = "Guardian";
            Debug.WriteLine( "Initial class selected: " + currentButtonClass );

            populateRadioButtons();
            populateButtonAssignments();
        }

        // The form closing event gets fired when we're exiting the program. We save our current state
        // into the permanent state and write the permanent state to the storage XML file. We update our
        // program settings (UI file directory and filename, initial class) and save them before we exit.
        private void ButtonAssignment_FormClosing( object sender, FormClosingEventArgs e )
        {
            Debug.WriteLine( "Button assignment form closing" );
            // Save as part of closing
            qrb.ButtonAssignments = buttonAssignments;
            qrb.saveDocument();

            // Update our settings
            if ( uicode.UIPath != Properties.Settings.Default.UIFileFolder )
                Properties.Settings.Default.UIFileFolder = uicode.UIPath;
            if ( uicode.UIFilename != Properties.Settings.Default.UIFilename )
                Properties.Settings.Default.UIFilename = uicode.UIFilename;
            Properties.Settings.Default.InitialClass = currentButtonClass;
            Properties.Settings.Default.Save();
        }

        // The exit button was clicked, so we exit the application.
        private void exitButton_Click( object sender, EventArgs e )
        {
            Application.Exit();
        }

        // Fires when the Generate UI File button was clicked. We save the current state into the permanent state
        // and open a standard file save dialog, setting it to default to the last directory and filename the user
        // generated UI code into. If they click OK in the file save dialog, we'll use the UI code object to generate
        // UI code from the QRBStorage object.
        private void generateButton_Click( object sender, EventArgs e )
        {
            Debug.WriteLine( "Generate UI code button clicked" );
            qrb.ButtonAssignments = buttonAssignments;
            buttonAssignments = qrb.CloneButtonAssignments();
            qrb.saveDocument();

            // Open a file save dialog, setting the path and filename based on the UIPath and UIFilename in the qrb object
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.InitialDirectory = uicode.UIPath;
            dlg.FileName = uicode.UIFilename;
            dlg.DefaultExt = ".txt";
            dlg.AddExtension = true;
            dlg.CheckPathExists = true;
            dlg.OverwritePrompt = true;
            dlg.ValidateNames = true;
            dlg.Filter = "Text files|*.txt|XML files|*.xml|All files|*.*";
            DialogResult result = dlg.ShowDialog( this );
            if ( result == DialogResult.OK )
            {
                string fp = dlg.FileName;
                Debug.Write( "UI code selected file: " + fp );
                uicode.UIPath = Path.GetDirectoryName( fp );
                Debug.Write( "UI code selected directory: " + uicode.UIPath );
                uicode.UIFilename = Path.GetFileName( fp );
                Debug.Write( "UI code selected filename: " + uicode.UIFilename );
                uicode.saveUICode( qrb );
            }
        }

        // Save our current state into the permanent state and save to disk, and create a new
        // current state cloned from our permanent state so future changes don't affect the
        // permanent state until the user saves it again.
        private void saveButton_Click( object sender, EventArgs e )
        {
            qrb.ButtonAssignments = buttonAssignments;
            buttonAssignments = qrb.CloneButtonAssignments();
            qrb.saveDocument();
        }

        // Revert changes. Throw away the current state, clone the permanent state and repopulate the
        // UI form elements.
        private void revertButton_Click( object sender, EventArgs e )
        {
            buttonAssignments = qrb.CloneButtonAssignments();
            populateButtonAssignments();
        }

        // When the Edit Spell Lines button is clicked, bring up the spell lines window handing it the QRBStorage
        // object and the current class so it can set itself to the class we're currently looking at. When it returns
        // control to us, repopulate the UI fields. The window code will have updated the QRBStorage object already.
        // The new spell lines window doesn't alter button assignments, so we don't mess with our current button
        // assignment state. We will, during repopulation, clean things up if the user deleted a currently-assigned
        // spell line.
        private void editSpellLinesButton_Click( object sender, EventArgs e )
        {
            using ( SpellLines sl_window = new SpellLines( qrb, currentButtonClass ) )
            {
                sl_window.ShowDialog( this );
                populateButtonAssignments();
            }
        }

        // When one of the class radio buttons is clicked, set our current class as it's name (the button names
        // match the classes) and repopulate the UI fields.
        private void classSelection_Click( object sender, EventArgs e )
        {
            RadioButton btn = sender as RadioButton;
            currentButtonClass = btn.Text;
            Debug.WriteLine( "New class selected: " + currentButtonClass );
            populateButtonAssignments();
        }

        // When the user selects a new value from one of the pull-down lists for button assignments, update
        // the current state for that button for our current class with the text of the combo box. We use the
        // Tag attribute of the combo boxes to map them to button numbers.
        private void buttonSpellLine_SelectionChanged( object sender, EventArgs e )
        {
            ComboBox spellLineName = sender as ComboBox;
            int button_number = int.Parse( spellLineName.Tag.ToString() );
            string line_name = spellLineName.Text;

            if ( buttonAssignments.ContainsKey( currentButtonClass ) )
                buttonAssignments[currentButtonClass].setButton( button_number, line_name );
        }

        // Show our About box when the user clicks the About button.
        private void aboutButton_Click( object sender, EventArgs e )
        {
            AboutBox a = new AboutBox();
            a.ShowDialog( this );
        }

        private void helpButton_Click( object sender, EventArgs e )
        {
            // Check QuickRaidButtons.hhp in the Hel Files folder for the topic IDs. You'll find what you need in the ALIAS and MAP sections
            Help.ShowHelp( this, "file:" + HelpFile, HelpNavigator.TopicId, "1001" );
        }
    }
}
